@@ -51,7 +51,7 @@ static int name_matches(char *found, char *required, char *homehost)
 }
 
 int Assemble(struct supertype *st, char *mddev, int mdfd,
-	     mddev_ident_t ident, char *conffile,
+	     mddev_ident_t ident,
 	     mddev_dev_t devlist, char *backup_file,
 	     int readonly, int runstop,
 	     char *update, char *homehost,
@@ -181,7 +181,7 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 		return 1;
 	}
 	if (devlist == NULL)
-		devlist = conf_get_devs(conffile);
+		devlist = conf_get_devs();
 	else if (mdfd >= 0)
 		inargv = 1;
 
@@ -421,6 +421,8 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			}
 			dfd = dev_open(devname, O_RDWR|O_EXCL);
 
+			remove_partitions(dfd);
+
 			if (super) {
 				free(super);
 				super = NULL;
@@ -451,13 +453,17 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 
 			if (strcmp(update, "uuid")==0 &&
 			    ident->bitmap_fd)
-				bitmap_update_uuid(ident->bitmap_fd, info.uuid);
+				if (bitmap_update_uuid(ident->bitmap_fd, info.uuid) != 0)
+					fprintf(stderr, Name ": Could not update uuid on %s.\n",
+						devname);
 		} else
 #endif
 		{
 			int dfd;
 			dfd = dev_open(devname, O_RDWR|O_EXCL);
 
+			remove_partitions(dfd);
+
 			if (super) {
 				free(super);
 				super = NULL;
@@ -497,6 +503,10 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 			if (nextspare < info.array.raid_disks)
 				nextspare = info.array.raid_disks;
 			i = nextspare++;
+		} else {
+			if (i >= info.array.raid_disks &&
+			    i >= nextspare)
+				nextspare = i+1;
 		}
 		if (i < 10000) {
 			if (i >= bestcnt) {
@@ -512,6 +522,28 @@ int Assemble(struct supertype *st, char *mddev, int mdfd,
 				best = newbest;
 				bestcnt = newbestcnt;
 			}
+			if (best[i] >=0 &&
+			    devices[best[i]].events == devices[devcnt].events &&
+			    devices[best[i]].minor != devices[devcnt].minor &&
+			    st->ss->major == 0 &&
+			    info.array.level != -4) {
+				/* two different devices with identical superblock.
+				 * Could be a mis-detection caused by overlapping
+				 * partitions.  fail-safe.
+				 */
+				fprintf(stderr, Name ": WARNING %s and %s appear"
+					" to have very similar superblocks.\n"
+					"      If they are really different, "
+					"please --zero the superblock on one\n"
+					"      If they are the same or overlap,"
+					" please remove one from %s.\n",
+					devices[best[i]].devname, devname,
+					inargv ? "the list" :
+					   "the\n      DEVICE list in mdadm.conf"
+					);
+				if (must_close) close(mdfd);
+				return 1;
+			}
 			if (best[i] == -1
 			    || devices[best[i]].events < devices[devcnt].events)
 				best[i] = devcnt;
